@@ -27,20 +27,18 @@ import (
 	"sync"
 	"time"
 
-	"go.opentelemetry.io/otel/attribute"
-	"go.opentelemetry.io/otel/trace"
-
 	"github.com/compose-spec/compose-go/v2/types"
 	"github.com/containerd/platforms"
-	"github.com/docker/compose/v2/internal/tracing"
-	moby "github.com/docker/docker/api/types"
 	containerType "github.com/docker/docker/api/types/container"
 	mmount "github.com/docker/docker/api/types/mount"
 	"github.com/docker/docker/api/types/versions"
 	specs "github.com/opencontainers/image-spec/specs-go/v1"
 	"github.com/sirupsen/logrus"
+	"go.opentelemetry.io/otel/attribute"
+	"go.opentelemetry.io/otel/trace"
 	"golang.org/x/sync/errgroup"
 
+	"github.com/docker/compose/v2/internal/tracing"
 	"github.com/docker/compose/v2/pkg/api"
 	"github.com/docker/compose/v2/pkg/progress"
 	"github.com/docker/compose/v2/pkg/utils"
@@ -49,7 +47,7 @@ import (
 const (
 	doubledContainerNameWarning = "WARNING: The %q service is using the custom container name %q. " +
 		"Docker requires each container to have a unique name. " +
-		"Remove the custom name to scale the service.\n"
+		"Remove the custom name to scale the service"
 )
 
 // convergence manages service's container lifecycle.
@@ -103,7 +101,7 @@ func (c *convergence) apply(ctx context.Context, project *types.Project, options
 
 		return tracing.SpanWrapFunc("service/apply", tracing.ServiceOptions(service), func(ctx context.Context) error {
 			strategy := options.RecreateDependencies
-			if utils.StringContains(options.Services, name) {
+			if slices.Contains(options.Services, name) {
 				strategy = options.Recreate
 			}
 			return c.ensureService(ctx, project, service, strategy, options.Inherit, options.Timeout)
@@ -112,6 +110,9 @@ func (c *convergence) apply(ctx context.Context, project *types.Project, options
 }
 
 func (c *convergence) ensureService(ctx context.Context, project *types.Project, service types.ServiceConfig, recreate string, inherit bool, timeout *time.Duration) error { //nolint:gocyclo
+	if service.Provider != nil {
+		return c.service.runPlugin(ctx, project, service, "up")
+	}
 	expected, err := getScale(service)
 	if err != nil {
 		return err
@@ -190,7 +191,6 @@ func (c *convergence) ensureService(ctx context.Context, project *types.Project,
 		case ContainerCreated:
 		case ContainerRestarting:
 		case ContainerExited:
-			w.Event(progress.CreatedEvent(name))
 		default:
 			container := container
 			eg.Go(tracing.EventWrapFuncForErrGroup(ctx, "service/start", tracing.ContainerOptions(container), func(ctx context.Context) error {
@@ -227,7 +227,9 @@ func (c *convergence) ensureService(ctx context.Context, project *types.Project,
 
 func (c *convergence) stopDependentContainers(ctx context.Context, project *types.Project, service types.ServiceConfig) error {
 	// Stop dependent containers, so they will be restarted after service is re-created
-	dependents := project.GetDependentsForService(service)
+	dependents := project.GetDependentsForService(service, func(dependency types.ServiceDependency) bool {
+		return dependency.Restart
+	})
 	if len(dependents) == 0 {
 		return nil
 	}
@@ -240,12 +242,12 @@ func (c *convergence) stopDependentContainers(ctx context.Context, project *type
 	}
 
 	for _, name := range dependents {
-		dependents := c.getObservedState(name)
-		for i, dependent := range dependents {
+		dependentStates := c.getObservedState(name)
+		for i, dependent := range dependentStates {
 			dependent.State = ContainerExited
-			dependents[i] = dependent
+			dependentStates[i] = dependent
 		}
-		c.setObservedState(name, dependents)
+		c.setObservedState(name, dependentStates)
 	}
 	return nil
 }
@@ -325,7 +327,7 @@ func (c *convergence) resolveSharedNamespaces(service *types.ServiceConfig) erro
 	return nil
 }
 
-func (c *convergence) mustRecreate(expected types.ServiceConfig, actual moby.Container, policy string) (bool, error) {
+func (c *convergence) mustRecreate(expected types.ServiceConfig, actual containerType.Summary, policy string) (bool, error) {
 	if policy == api.RecreateNever {
 		return false, nil
 	}
@@ -357,7 +359,7 @@ func (c *convergence) mustRecreate(expected types.ServiceConfig, actual moby.Con
 	return false, nil
 }
 
-func checkExpectedNetworks(expected types.ServiceConfig, actual moby.Container, networks map[string]string) bool {
+func checkExpectedNetworks(expected types.ServiceConfig, actual containerType.Summary, networks map[string]string) bool {
 	// check the networks container is connected to are the expected ones
 	for net := range expected.Networks {
 		id := networks[net]
@@ -380,7 +382,7 @@ func checkExpectedNetworks(expected types.ServiceConfig, actual moby.Container,
 	return false
 }
 
-func checkExpectedVolumes(expected types.ServiceConfig, actual moby.Container, volumes map[string]string) bool {
+func checkExpectedVolumes(expected types.ServiceConfig, actual containerType.Summary, volumes map[string]string) bool {
 	// check container's volume mounts and search for the expected ones
 	for _, vol := range expected.Volumes {
 		if vol.Type != string(mmount.TypeVolume) {
@@ -420,22 +422,22 @@ func getDefaultContainerName(projectName, serviceName, index string) string {
 	return strings.Join([]string{projectName, serviceName, index}, api.Separator)
 }
 
-func getContainerProgressName(container moby.Container) string {
-	return "Container " + getCanonicalContainerName(container)
+func getContainerProgressName(ctr containerType.Summary) string {
+	return "Container " + getCanonicalContainerName(ctr)
 }
 
 func containerEvents(containers Containers, eventFunc func(string) progress.Event) []progress.Event {
 	events := []progress.Event{}
-	for _, container := range containers {
-		events = append(events, eventFunc(getContainerProgressName(container)))
+	for _, ctr := range containers {
+		events = append(events, eventFunc(getContainerProgressName(ctr)))
 	}
 	return events
 }
 
 func containerReasonEvents(containers Containers, eventFunc func(string, string) progress.Event, reason string) []progress.Event {
 	events := []progress.Event{}
-	for _, container := range containers {
-		events = append(events, eventFunc(getContainerProgressName(container), reason))
+	for _, ctr := range containers {
+		events = append(events, eventFunc(getContainerProgressName(ctr), reason))
 	}
 	return events
 }
@@ -565,7 +567,7 @@ func shouldWaitForDependency(serviceName string, dependencyConfig types.ServiceD
 	return true, nil
 }
 
-func nextContainerNumber(containers []moby.Container) int {
+func nextContainerNumber(containers []containerType.Summary) int {
 	maxNumber := 0
 	for _, c := range containers {
 		s, ok := c.Labels[api.ContainerNumberLabel]
@@ -586,12 +588,19 @@ func nextContainerNumber(containers []moby.Container) int {
 
 func (s *composeService) createContainer(ctx context.Context, project *types.Project, service types.ServiceConfig,
 	name string, number int, opts createOptions,
-) (container moby.Container, err error) {
+) (ctr containerType.Summary, err error) {
 	w := progress.ContextWriter(ctx)
 	eventName := "Container " + name
 	w.Event(progress.CreatingEvent(eventName))
-	container, err = s.createMobyContainer(ctx, project, service, name, number, nil, opts, w)
+	ctr, err = s.createMobyContainer(ctx, project, service, name, number, nil, opts, w)
 	if err != nil {
+		if ctx.Err() == nil {
+			w.Event(progress.Event{
+				ID:         eventName,
+				Status:     progress.Error,
+				StatusText: err.Error(),
+			})
+		}
 		return
 	}
 	w.Event(progress.CreatedEvent(eventName))
@@ -599,18 +608,27 @@ func (s *composeService) createContainer(ctx context.Context, project *types.Pro
 }
 
 func (s *composeService) recreateContainer(ctx context.Context, project *types.Project, service types.ServiceConfig,
-	replaced moby.Container, inherit bool, timeout *time.Duration,
-) (moby.Container, error) {
-	var created moby.Container
+	replaced containerType.Summary, inherit bool, timeout *time.Duration,
+) (created containerType.Summary, err error) {
 	w := progress.ContextWriter(ctx)
-	w.Event(progress.NewEvent(getContainerProgressName(replaced), progress.Working, "Recreate"))
+	eventName := getContainerProgressName(replaced)
+	w.Event(progress.NewEvent(eventName, progress.Working, "Recreate"))
+	defer func() {
+		if err != nil && ctx.Err() == nil {
+			w.Event(progress.Event{
+				ID:         eventName,
+				Status:     progress.Error,
+				StatusText: err.Error(),
+			})
+		}
+	}()
 
 	number, err := strconv.Atoi(replaced.Labels[api.ContainerNumberLabel])
 	if err != nil {
 		return created, err
 	}
 
-	var inherited *moby.Container
+	var inherited *containerType.Summary
 	if inherit {
 		inherited = &replaced
 	}
@@ -643,18 +661,23 @@ func (s *composeService) recreateContainer(ctx context.Context, project *types.P
 		return created, err
 	}
 
-	w.Event(progress.NewEvent(getContainerProgressName(replaced), progress.Done, "Recreated"))
+	w.Event(progress.NewEvent(eventName, progress.Done, "Recreated"))
 	return created, err
 }
 
-func (s *composeService) startContainer(ctx context.Context, container moby.Container) error {
+// force sequential calls to ContainerStart to prevent race condition in engine assigning ports from ranges
+var startMx sync.Mutex
+
+func (s *composeService) startContainer(ctx context.Context, ctr containerType.Summary) error {
 	w := progress.ContextWriter(ctx)
-	w.Event(progress.NewEvent(getContainerProgressName(container), progress.Working, "Restart"))
-	err := s.apiClient().ContainerStart(ctx, container.ID, containerType.StartOptions{})
+	w.Event(progress.NewEvent(getContainerProgressName(ctr), progress.Working, "Restart"))
+	startMx.Lock()
+	defer startMx.Unlock()
+	err := s.apiClient().ContainerStart(ctx, ctr.ID, containerType.StartOptions{})
 	if err != nil {
 		return err
 	}
-	w.Event(progress.NewEvent(getContainerProgressName(container), progress.Done, "Restarted"))
+	w.Event(progress.NewEvent(getContainerProgressName(ctr), progress.Done, "Restarted"))
 	return nil
 }
 
@@ -663,11 +686,11 @@ func (s *composeService) createMobyContainer(ctx context.Context,
 	service types.ServiceConfig,
 	name string,
 	number int,
-	inherit *moby.Container,
+	inherit *containerType.Summary,
 	opts createOptions,
 	w progress.Writer,
-) (moby.Container, error) {
-	var created moby.Container
+) (containerType.Summary, error) {
+	var created containerType.Summary
 	cfgs, err := s.getCreateConfigs(ctx, project, service, number, inherit, opts)
 	if err != nil {
 		return created, err
@@ -701,11 +724,11 @@ func (s *composeService) createMobyContainer(ctx context.Context,
 	if err != nil {
 		return created, err
 	}
-	created = moby.Container{
+	created = containerType.Summary{
 		ID:     inspectedContainer.ID,
 		Labels: inspectedContainer.Config.Labels,
 		Names:  []string{inspectedContainer.Name},
-		NetworkSettings: &moby.SummaryNetworkSettings{
+		NetworkSettings: &containerType.NetworkSettingsSummary{
 			Networks: inspectedContainer.NetworkSettings.Networks,
 		},
 	}
@@ -822,11 +845,11 @@ func (s *composeService) isServiceHealthy(ctx context.Context, containers Contai
 			return false, fmt.Errorf("container %s has no healthcheck configured", name)
 		}
 		switch container.State.Health.Status {
-		case moby.Healthy:
+		case containerType.Healthy:
 			// Continue by checking the next container.
-		case moby.Unhealthy:
+		case containerType.Unhealthy:
 			return false, fmt.Errorf("container %s is unhealthy", name)
-		case moby.Starting:
+		case containerType.Starting:
 			return false, nil
 		default:
 			return false, fmt.Errorf("container %s had unexpected health status %q", name, container.State.Health.Status)
@@ -870,19 +893,19 @@ func (s *composeService) startService(ctx context.Context,
 	}
 
 	w := progress.ContextWriter(ctx)
-	for _, container := range containers.filter(isService(service.Name)) {
-		if container.State == ContainerRunning {
+	for _, ctr := range containers.filter(isService(service.Name)) {
+		if ctr.State == ContainerRunning {
 			continue
 		}
-		eventName := getContainerProgressName(container)
+		eventName := getContainerProgressName(ctr)
 		w.Event(progress.StartingEvent(eventName))
-		err = s.apiClient().ContainerStart(ctx, container.ID, containerType.StartOptions{})
+		err = s.apiClient().ContainerStart(ctx, ctr.ID, containerType.StartOptions{})
 		if err != nil {
 			return err
 		}
 
 		for _, hook := range service.PostStart {
-			err = s.runHook(ctx, container, service, hook, listener)
+			err = s.runHook(ctx, ctr, service, hook, listener)
 			if err != nil {
 				return err
 			}
