@@ -22,8 +22,7 @@ import (
 	"io"
 	"time"
 
-	"github.com/docker/docker/api/types"
-	containerType "github.com/docker/docker/api/types/container"
+	"github.com/docker/docker/api/types/container"
 	"github.com/docker/docker/errdefs"
 	"github.com/docker/docker/pkg/stdcopy"
 	"github.com/sirupsen/logrus"
@@ -43,11 +42,11 @@ func (s *composeService) Logs(
 	var err error
 
 	if options.Index > 0 {
-		container, err := s.getSpecifiedContainer(ctx, projectName, oneOffExclude, true, options.Services[0], options.Index)
+		ctr, err := s.getSpecifiedContainer(ctx, projectName, oneOffExclude, true, options.Services[0], options.Index)
 		if err != nil {
 			return err
 		}
-		containers = append(containers, container)
+		containers = append(containers, ctr)
 	} else {
 		containers, err = s.getContainers(ctx, projectName, oneOffExclude, true, options.Services...)
 		if err != nil {
@@ -92,7 +91,7 @@ func (s *composeService) Logs(
 		}
 
 		eg.Go(func() error {
-			err := s.watchContainers(ctx, projectName, options.Services, nil, printer.HandleEvent, containers, func(c types.Container, t time.Time) error {
+			err := s.watchContainers(ctx, projectName, options.Services, nil, printer.HandleEvent, containers, func(c container.Summary, t time.Time) error {
 				printer.HandleEvent(api.ContainerEvent{
 					Type:      api.ContainerEventAttach,
 					Container: getContainerNameWithoutProject(c),
@@ -115,7 +114,7 @@ func (s *composeService) Logs(
 					return err
 				})
 				return nil
-			}, func(c types.Container, t time.Time) error {
+			}, func(c container.Summary, t time.Time) error {
 				printer.HandleEvent(api.ContainerEvent{
 					Type:      api.ContainerEventAttach,
 					Container: "", // actual name will be set by start event
@@ -132,13 +131,13 @@ func (s *composeService) Logs(
 	return eg.Wait()
 }
 
-func (s *composeService) logContainers(ctx context.Context, consumer api.LogConsumer, c types.Container, options api.LogOptions) error {
+func (s *composeService) logContainers(ctx context.Context, consumer api.LogConsumer, c container.Summary, options api.LogOptions) error {
 	cnt, err := s.apiClient().ContainerInspect(ctx, c.ID)
 	if err != nil {
 		return err
 	}
 
-	r, err := s.apiClient().ContainerLogs(ctx, cnt.ID, containerType.LogsOptions{
+	r, err := s.apiClient().ContainerLogs(ctx, cnt.ID, container.LogsOptions{
 		ShowStdout: true,
 		ShowStderr: true,
 		Follow:     options.Follow,
